@@ -32,20 +32,22 @@ detectionHistory <- function(recordTable,
   stopifnot(hasArg(camOp))
 
   stopifnot(hasArg(stationCol))
+  stopifnot(stationCol %in% colnames(recordTable))
   stopifnot(length(stationCol) == 1)
   recordTable[,stationCol] <- as.character(recordTable[,stationCol])
   stopifnot(is.character(stationCol))
 
-
   stopifnot(hasArg(speciesCol))
+  stopifnot(speciesCol %in% colnames(recordTable))
   stopifnot(length(speciesCol) == 1)
   recordTable[,speciesCol] <- as.character(recordTable[,speciesCol])
   stopifnot(is.character(speciesCol))
 
 
   stopifnot(hasArg(recordDateTimeCol))
+  stopifnot(recordDateTimeCol %in% colnames(recordTable))
   stopifnot(length(recordDateTimeCol) == 1)
-  recordTable[,recordDateTimeCol] <- as.character(recordTable[,recordDateTimeCol])
+  recordTable[,recordDateTimeCol] <- as.character(recordTable[,recordDateTimeCol])   # make character to get rid of attributes. Will later assign time zone again
   stopifnot(is.character(recordDateTimeCol))
 
 
@@ -69,48 +71,41 @@ detectionHistory <- function(recordTable,
 
   if(hasArg(maxNumberDays)){
     maxNumberDays <- as.integer(maxNumberDays)
-    if(maxNumberDays > ncol(camOp)) stop("maxNumberDays must be smaller than the number of columns of camOp")
+    if(maxNumberDays > ncol(camOp)) stop("maxNumberDays is larger than the number of columns of camOp")
     if(maxNumberDays < occasionLength) stop("maxNumberDays must be larger than or equal to occasionLength")
   }
-  
+
   if(hasArg(buffer)) {
     stopifnot(is.numeric(buffer))
     buffer <- round(buffer)
     stopifnot(buffer >= 1)
   }
 
-
   stopifnot(c(speciesCol, recordDateTimeCol, stationCol) %in% colnames(recordTable))
 
-
   if(species %in% recordTable[,speciesCol] == FALSE) stop("species is not in speciesCol of recordTable")
 
   if(writecsv == TRUE){
     if(file.exists(outDir) == FALSE){stop("outDir does not exist")}
   }
 
-
   if(includeEffort == TRUE){
     if(!hasArg(scaleEffort)) stop("scaleEffort must be defined if includeEffort is TRUE")
     if(class(scaleEffort) != "logical") stop("scaleEffort must be logical (TRUE or FALSE)")
   } else {scaleEffort <- FALSE}
 
-
   #############
   # bring date, time, station ids into shape
 
   subset_species           <- subset(recordTable, recordTable[,speciesCol] == species)
-  subset_species$DateTime2 <- strptime(subset_species[,recordDateTimeCol], tz = timeZone, format = recordDateTimeFormat)
-  subset_species$Date2     <- as.Date(subset_species$DateTime2)
-
+  subset_species$DateTime2 <- as.POSIXlt(subset_species[,recordDateTimeCol], tz = timeZone, format = recordDateTimeFormat)
 
   # check consistency of argument day1
   stopifnot(class(day1) == "character")
   if(day1 == "survey") {day1switch <- 1} else {
     if(day1 == "station") {day1switch <- 2} else {
       try(date.test <- as.Date(day1), silent = TRUE)
-      if(class(date.test) != "Date") stop("could not interpret argument day1: can only be 'station', 'survey' or a specific date")
-      if(all(subset_species$Date2 < as.Date(day1))) stop(paste("there are no records after your specified day1:", day1), call. = FALSE)
+      if(class(date.test) != "Date") stop('could not interpret argument day1: can only be "station", "survey" or a specific date (e.g. "2015-12-31")')
       if(hasArg(buffer)) stop("if buffer is defined, day1 can only be 'survey' or 'station'")
       suppressWarnings(rm(date.test))
       day1switch <- 3
@@ -121,7 +116,6 @@ detectionHistory <- function(recordTable,
   if(any(is.na(subset_species$DateTime2))) stop("at least 1 entry in recordDateTimeCol of recordTable could not be interpreted using recordDateTimeFormat")
 
 
-  ####
   cam.op.worked0 <- as.matrix(camOp)
 
   if(all(as.character(unique(subset_species[,stationCol])) %in% rownames(cam.op.worked0)) == FALSE){
@@ -130,64 +124,24 @@ detectionHistory <- function(recordTable,
 
   ################################################
   # compute date range of stations and records
+  arg.list0 <- list(cam.op = cam.op.worked0, subset_species_tmp = subset_species, stationCol_tmp = stationCol, day1_tmp = day1, occasionStartTime_tmp = occasionStartTime, timeZone_tmp = timeZone)
 
-  cam.tmp.min <- apply(cam.op.worked0, MARGIN = 1, function(X){min(which(!is.na(X)))})    # 1st day of each station
-  cam.tmp.max <- apply(cam.op.worked0, MARGIN = 1, function(X){max(which(!is.na(X)))})    # last day of each station
+    if(hasArg(maxNumberDays))  arg.list0 <- c(arg.list0,   maxNumberDays_tmp = maxNumberDays)
+    if(hasArg(buffer))   arg.list0 <- c(arg.list0, buffer_tmp =  buffer)
 
-  rec.tmp.min  <- aggregate(subset_species$Date2,
-                            list(subset_species[,stationCol]),
-                            FUN = min)
-  rec.tmp.max  <- aggregate(subset_species$Date2,
-                            list(subset_species[,stationCol]),
-                            FUN = max)
-
-
-  date_ranges <- data.frame(rec.min = rec.tmp.min[match(rownames(cam.op.worked0), rec.tmp.min[,1]), 2],   # as.Date(unlist(rec.tmp.min), origin = "1970-01-01"),
-                            rec.max = rec.tmp.max[match(rownames(cam.op.worked0), rec.tmp.max[,1]), 2],   # as.Date(unlist(rec.tmp.max), origin = "1970-01-01"),
-                            cam.min = as.Date(colnames(cam.op.worked0)[cam.tmp.min]),
-                            cam.max = as.Date(colnames(cam.op.worked0)[cam.tmp.max])
-  )
-  date_ranges$diff.days <- date_ranges$cam.max - date_ranges$cam.min
-  rownames(date_ranges) <- rownames(cam.op.worked0)
-
-  # check if images were taken between setup and retrieval dates (Error if images outside station date range)
-  if(any(date_ranges$rec.min < date_ranges$cam.min | date_ranges$rec.max > date_ranges$cam.max, na.rm = TRUE)) stop(paste("record date outside camera operation date range: ",
-                                                                                                                          paste(rownames(date_ranges)[which(date_ranges$rec.min < date_ranges$cam.min)], collapse = ", " )))
-
-  # adjust camera operation dates with buffer and
-  # remove records from stations that were operational for a shorter time than the buffer period
-  if(hasArg(buffer)){
-    if(any(date_ranges$diff.days <= buffer)){
-      warning(paste("buffer (", buffer, ") is larger or or equal to date range of cameras: ", paste(rownames(date_ranges)[which(date_ranges$diff.days <= buffer)], collapse = ", ")))
-    }
-    date_ranges$cam.min <- date_ranges$cam.min + buffer
-    if(any(date_ranges$cam.min >= date_ranges$cam.max)){
-      stations2remove <- which(date_ranges$cam.min >= date_ranges$cam.max)
-      if(length(stations2remove) == nrow(cam.op.worked0)) stop("No station left. your buffer argument is too large.", call. = FALSE)
-      if(any(subset_species[,stationCol] %in% rownames(cam.op.worked0) [stations2remove])){
-        subset_species <- subset_species[-which(subset_species[,stationCol] %in% rownames(cam.op.worked0) [stations2remove]),]
-      }
-      if(nrow(subset_species) == 0){stop("No more records left. Choose a smaller buffer argument", call. = FALSE)}
-       cam.op.worked0 [stations2remove,] <- NA
-       rm(stations2remove)
-    }
-  }
-
-  rm(cam.tmp.min, cam.tmp.max, rec.tmp.min, rec.tmp.max)
+  date_ranges <- do.call(createDateRangeTable, arg.list0)
 
+  rm(arg.list0)
 
 #######################
 # adjust camera operation matrix
 
-  arg.list0 <- list(cam.op = cam.op.worked0, date_ranges2 = date_ranges, day1_2 = day1, occasionStartTime2 = occasionStartTime)
-
-  if(!hasArg(maxNumberDays) & !hasArg(buffer))  cam.op.worked <- do.call(adjustCameraOperationMatrix, arg.list0)
-  if(hasArg(maxNumberDays)  & !hasArg(buffer))  cam.op.worked <- do.call(adjustCameraOperationMatrix, c(arg.list0, maxNumberDays = maxNumberDays))
-  if(!hasArg(maxNumberDays) & hasArg(buffer))   cam.op.worked <- do.call(adjustCameraOperationMatrix, c(arg.list0, buffer =  buffer))
-  if(hasArg(maxNumberDays)  & hasArg(buffer))   cam.op.worked <- do.call(adjustCameraOperationMatrix, c(arg.list0, maxNumberDays = maxNumberDays, buffer =  buffer))
-
-  rm(arg.list0)
+  cam.op.worked <- adjustCameraOperationMatrix(cam.op = cam.op.worked0, date_ranges2 = date_ranges, timeZone_tmp = timeZone, day1_2 = day1)
 
+  # append occasionStartTime (if != 0) to column names for output table
+  if(occasionStartTime != 0){
+    colnames(cam.op.worked) <- paste(colnames(cam.op.worked), "+", occasionStartTime, "h", sep = "")
+  }
 
 ######################
 # calculate trapping effort by station and occasion
@@ -196,44 +150,29 @@ detectionHistory <- function(recordTable,
   effort <- effort.tmp[[1]]
   if(isTRUE(scaleEffort))  scale.eff.tmp.attr <- effort.tmp[[2]]
 
-
 ###################
 # remove records that fall into buffer period or were taken after maxNumberDays
-  arg.list0 <- list(subset_species2 = subset_species, stationCol2 = stationCol, date_ranges2 = date_ranges, occasionStartTime2 = occasionStartTime, timeZone2 = timeZone)
-
-  if(!hasArg(maxNumberDays) & !hasArg(buffer))  subset_species <- do.call(cleanSubsetSpecies, arg.list0)
-  if(hasArg(maxNumberDays)  & !hasArg(buffer))  subset_species <- do.call(cleanSubsetSpecies, c(arg.list0, maxNumberDays = maxNumberDays))
-  if(!hasArg(maxNumberDays) & hasArg(buffer))   subset_species <- do.call(cleanSubsetSpecies, c(arg.list0, buffer =  buffer))
-  if(hasArg(maxNumberDays)  & hasArg(buffer))   subset_species <- do.call(cleanSubsetSpecies, c(arg.list0, maxNumberDays = maxNumberDays, buffer = buffer))
-
-  rm(arg.list0)
 
+  subset_species <- cleanSubsetSpecies(subset_species2 = subset_species, stationCol2 = stationCol, date_ranges2 = date_ranges)
 
 ############
   #  define the 1st day of the effective survey period.
+if(day1 %in% c("survey")){
+  time2 <- date_ranges$start_first_occasion_survey[match(subset_species[,stationCol], rownames(date_ranges))]
+} else {
+  time2 <- date_ranges$start_first_occasion[match(subset_species[,stationCol], rownames(date_ranges))]
+}
 
-  if(day1 == "survey")  {    # same starting day for all stations
-      time2 = rep(as.POSIXct(paste(as.character(min(date_ranges$cam.min)), "00:00:00"), tz = timeZone), times = nrow(subset_species))
-  } else {
-    if(day1 == "station") {  # individual starting days for all stations
-        time2 = as.POSIXct(paste(as.character(date_ranges$cam.min[match(subset_species[,stationCol], rownames(date_ranges))]), "00:00:00"), tz = timeZone)
-    } else {
-      if(class(as.Date(day1)) == "Date") {   #  some specific date
-        time2 = rep(as.POSIXct( paste(as.Date(day1), "00:00:00"), tz = timeZone), times = nrow(subset_species))
-      }
-    }
-  }
-
-
-  # calculate time difference between records and first day of detection history (the occasion each record belongs into)
-
-  subset_species$occasion <- as.numeric(ceiling((difftime(time1  = subset_species[,recordDateTimeCol],
+  # calculate the occasion each record belongs into from the time difference between records and beginning of the first occasion
+  subset_species$occasion <- as.numeric(ceiling((difftime(time1  = subset_species$DateTime2,
                                                           time2 =  time2,
                                                           units = "secs",
                                                           tz = timeZone)
-                                                 - occasionStartTime * 3600) / (occasionLength * 86400)))
+                                                / (occasionLength * 86400))))
 
 
+ if(max(subset_species$occasion) > ncol(effort)) {stop("encountered a bug. I'm Sorry. Please report it.")}
+
   ############
   # make detection history
 
@@ -245,6 +184,8 @@ detectionHistory <- function(recordTable,
     occasions.by.station <- tapply(X = subset_species$occasion, INDEX = subset_species[,stationCol], FUN = unique, simplify = FALSE)
 
     for(xyz in which(sapply(occasions.by.station, FUN = function(x){!is.null(x)}))){
+      if(any(occasions.by.station[[xyz]] < 0)) stop("this is a bug in the function (line 181). Please report it.", call. = FALSE)
+      if(any(occasions.by.station[[xyz]] > ncol(record.hist))) stop("this is a bug in the function (line 182). Please report it.", call. = FALSE)
       record.hist[match(names(occasions.by.station)[xyz], rownames(record.hist)), occasions.by.station[[xyz]]] <- 1
     }
     record.hist[is.na(cam.op.worked)] <- NA   # remove the records that were taken when cams were NA (redundant with above:   # remove records taken after day1 + maxNumberDays)
@@ -263,32 +204,40 @@ detectionHistory <- function(recordTable,
     record.hist[is.na(effort)] <- NA
   }
 
-
-
-  # rownames of output
+# assign row names to output
   row.names(record.hist) <- row.names(effort) <- rownames(cam.op.worked)
 
-  # column names for output table
-  colnames(cam.op.worked) <- paste(colnames(cam.op.worked), "+", occasionStartTime, "h", sep = "")
 
 # assign column names
   if(isTRUE(datesAsOccasionNames)){
-    if(occasionLength == 1){
-      colnames.tmp <- colnames(cam.op.worked)
-    } else {
+
       seq.tmp <- seq(from = 1, by = occasionLength, length.out = ncol(record.hist))
-      colnames.tmp <- paste(colnames(cam.op.worked)[seq.tmp],
-                            colnames(cam.op.worked)[seq.tmp + occasionLength - 1], sep = "_")
-      colnames.tmp[length(colnames.tmp)] <- paste(colnames(cam.op.worked)[max(seq.tmp)],      # fix the last one
-                                                  colnames(cam.op.worked)[ncol(cam.op.worked)],
-                                                  sep = "_")
-    }
+      if(occasionStartTime != 0){
+        colnames.tmp <- paste(colnames(cam.op.worked)[seq.tmp],
+                              colnames(cam.op.worked)[seq.tmp + occasionLength], sep = "_")
+      } else {
+        colnames.tmp <- paste(colnames(cam.op.worked)[seq.tmp],
+                              colnames(cam.op.worked)[seq.tmp + occasionLength - 1], sep = "_")
+      }
+      # make sure the last occasion ends on the last day
+
+      if(day1 == "station"){
+        colnames.tmp[length(colnames.tmp)] <- paste(colnames(cam.op.worked)[max(seq.tmp)],
+                                                    colnames(cam.op.worked)[ncol(cam.op.worked)],
+                                                    sep = "_")
+      } else {   # if day1 = "survey" or date
+        colnames.tmp[length(colnames.tmp)] <- paste(colnames(cam.op.worked)[max(seq.tmp)],
+                                                          max(as.Date(date_ranges$end_last_occasion)),
+                                                    sep = "_")
+        }
+
     colnames(record.hist) <- colnames(effort) <- colnames.tmp
 
   }  else {
     colnames(record.hist) <- colnames(effort) <-  paste("o", seq(1,ncol(record.hist), by = 1), sep = "")
   }
 
+
   ################################################
   # save output as table
   if(day1switch == 1) day1string <- "_first_day_from_survey"
@@ -303,6 +252,7 @@ detectionHistory <- function(recordTable,
     scaleEffortstring <- ""
   }
 
+  # create names for the csv files
   outtable.name <- paste(species, "__record_history__", effortstring,
                          occasionLength, "_days_per_occasion_",
                          maxNumberDaysstring,
@@ -338,6 +288,7 @@ detectionHistory <- function(recordTable,
       }
     }
   }
+
   if(isTRUE(includeEffort)){
     if(scaleEffort == TRUE){
       return(list(detection_history = record.hist,
