@@ -45,21 +45,25 @@ spatialDetectionHistory <- function(recordTableIndividual,
 
 
   stopifnot(hasArg(stationCol))
+  stopifnot(stationCol %in% colnames(recordTableIndividual))
   stopifnot(length(stationCol) == 1)
   recordTableIndividual[,stationCol] <- as.character(recordTableIndividual[,stationCol])
   stopifnot(is.character(stationCol))
 
   stopifnot(hasArg(speciesCol))
+  stopifnot(speciesCol %in% colnames(recordTableIndividual))
   stopifnot(length(speciesCol) == 1)
   recordTableIndividual[,speciesCol] <- as.character(recordTableIndividual[,speciesCol])
   stopifnot(is.character(speciesCol))
 
   stopifnot(hasArg(individualCol))
+  stopifnot(individualCol %in% colnames(recordTableIndividual))
   stopifnot(length(individualCol) == 1)
   recordTableIndividual[,individualCol] <- as.character(recordTableIndividual[,individualCol])
   stopifnot(is.character(individualCol))
 
   stopifnot(hasArg(recordDateTimeCol))
+  stopifnot(recordDateTimeCol %in% colnames(recordTableIndividual))
   stopifnot(length(recordDateTimeCol) == 1)
   recordTableIndividual[,recordDateTimeCol] <- as.character(recordTableIndividual[,recordDateTimeCol])
   stopifnot(is.character(recordDateTimeCol))
@@ -131,10 +135,8 @@ spatialDetectionHistory <- function(recordTableIndividual,
   #####################################################################################################################
   # bring date, time, station ids into shape
 
-  subset_species <- subset(recordTableIndividual, recordTableIndividual[,speciesCol] == species)
-
-  subset_species$DateTime2 <- strptime(subset_species[,recordDateTimeCol], tz = timeZone, format = recordDateTimeFormat)
-  subset_species$Date2 <- as.Date(subset_species$DateTime2)
+  subset_species           <- subset(recordTableIndividual, recordTableIndividual[,speciesCol] == species)
+  subset_species$DateTime2 <- as.POSIXlt(subset_species[,recordDateTimeCol], tz = timeZone, format = recordDateTimeFormat)
 
   # check consistency of argument day1
   stopifnot(class(day1) == "character")
@@ -142,9 +144,8 @@ spatialDetectionHistory <- function(recordTableIndividual,
   if(day1 == "survey") {day1switch <- 1} else {
     if(day1 == "station") {day1switch <- 2} else {
       try(date.test <- as.Date(day1), silent = TRUE)
-      if(class(date.test) != "Date") stop("could not interpret argument day1: can only be 'station', 'survey' or a specific date")
-      if(all(subset_species$Date2 < as.Date(day1))) stop(paste("there are no records after your specified day1:", day1), call. = FALSE)
-      if(hasArg(buffer)) stop("if buffer is defined, day1 can only be 'survey' or 'station'")
+       if(class(date.test) != "Date") stop('could not interpret argument day1: can only be "station", "survey" or a specific date (e.g. "2015-12-31")')
+       if(hasArg(buffer)) stop("if buffer is defined, day1 can only be 'survey' or 'station'")
       suppressWarnings(rm(date.test))
       day1switch <- 3
     }
@@ -165,63 +166,24 @@ spatialDetectionHistory <- function(recordTableIndividual,
   ################################################
   # compute date range of stations and records
 
-  cam.tmp.min <- apply(cam.op.worked0, MARGIN = 1, function(X){min(which(!is.na(X)))})    # 1st day of each station
-  cam.tmp.max <- apply(cam.op.worked0, MARGIN = 1, function(X){max(which(!is.na(X)))})    # last day of each station
+  arg.list0 <- list(cam.op = cam.op.worked0, subset_species_tmp = subset_species, stationCol_tmp = stationCol, day1_tmp = day1, occasionStartTime_tmp = occasionStartTime, timeZone_tmp = timeZone)
 
-  rec.tmp.min  <- aggregate(subset_species$Date2,
-                            list(subset_species[,stationCol]),
-                            FUN = min)
-  rec.tmp.max  <- aggregate(subset_species$Date2,
-                            list(subset_species[,stationCol]),
-                            FUN = max)
+    if(hasArg(maxNumberDays))  arg.list0 <- c(arg.list0,   maxNumberDays_tmp = maxNumberDays)
+    if(hasArg(buffer))   arg.list0 <- c(arg.list0, buffer_tmp =  buffer)
 
+  date_ranges <- do.call(createDateRangeTable, arg.list0)
 
-  date_ranges <- data.frame(rec.min = rec.tmp.min[match(rownames(cam.op.worked0), rec.tmp.min[,1]), 2],
-                            rec.max = rec.tmp.max[match(rownames(cam.op.worked0), rec.tmp.max[,1]), 2],
-                            cam.min = as.Date(colnames(cam.op.worked0)[cam.tmp.min]),
-                            cam.max = as.Date(colnames(cam.op.worked0)[cam.tmp.max])
-  )
-  date_ranges$diff.days <- date_ranges$cam.max - date_ranges$cam.min
-  rownames(date_ranges) <- rownames(cam.op.worked0)
-
-  # check if images were taken between setup and retrieval dates (Error if images outside station date range)
-  if(any(date_ranges$rec.min < date_ranges$cam.min | date_ranges$rec.max > date_ranges$cam.max, na.rm = TRUE)) stop(paste("record date outside camera operation date range: ",
-                                                                                                                          paste(rownames(date_ranges)[which(date_ranges$rec.min < date_ranges$cam.min)], collapse = ", " )))
-
-  # adjust camera operation dates with buffer and
-  # remove records from stations that were operational for a shorter time than the buffer period
-  if(hasArg(buffer)){
-    if(any(date_ranges$diff.days <= buffer)){
-      warning(paste("buffer (", buffer, ") is larger or or equal to date range of cameras: ", paste(rownames(date_ranges)[which(date_ranges$diff.days <= buffer)], collapse = ", ")))
-    }
-    date_ranges$cam.min <- date_ranges$cam.min + buffer
-    if(any(date_ranges$cam.min >= date_ranges$cam.max)){
-      stations2remove <- which(date_ranges$cam.min >= date_ranges$cam.max)
-      if(length(stations2remove) == nrow(cam.op.worked0)) stop("No station left. your buffer argument is too large.", call. = FALSE)
-      if(any(subset_species[,stationCol] %in% rownames(cam.op.worked0) [stations2remove])){
-        subset_species <- subset_species[-which(subset_species[,stationCol] %in% rownames(cam.op.worked0) [stations2remove]),]
-      }
-      if(nrow(subset_species) == 0){stop("No more records left. Choose a smaller buffer argument", call. = FALSE)}
-      cam.op.worked0 [stations2remove,] <- NA
-      rm(stations2remove)
-    }
-  }
-
-  rm(cam.tmp.min, cam.tmp.max, rec.tmp.min, rec.tmp.max)
-
+  rm(arg.list0)
 
   #######################
   # adjust camera operation matrix
 
-  arg.list0 <- list(cam.op = cam.op.worked0, date_ranges2 = date_ranges, day1_2 = day1, occasionStartTime2 = occasionStartTime)
-
-  if(!hasArg(maxNumberDays) & !hasArg(buffer))  cam.op.worked <- do.call(adjustCameraOperationMatrix, arg.list0)
-  if(hasArg(maxNumberDays)  & !hasArg(buffer))  cam.op.worked <- do.call(adjustCameraOperationMatrix, c(arg.list0, maxNumberDays = maxNumberDays))
-  if(!hasArg(maxNumberDays) & hasArg(buffer))   cam.op.worked <- do.call(adjustCameraOperationMatrix, c(arg.list0, buffer =  buffer))
-  if(hasArg(maxNumberDays)  & hasArg(buffer))   cam.op.worked <- do.call(adjustCameraOperationMatrix, c(arg.list0, maxNumberDays = maxNumberDays, buffer =  buffer))
-
-  rm(arg.list0)
+  cam.op.worked <- adjustCameraOperationMatrix(cam.op = cam.op.worked0, date_ranges2 = date_ranges, timeZone_tmp = timeZone, day1_2 = day1)
 
+  # append occasionStartTime (if != 0) to column names for output table
+  if(occasionStartTime != 0){
+    colnames(cam.op.worked) <- paste(colnames(cam.op.worked), "+", occasionStartTime, "h", sep = "")
+  }
 
   ######################
   # calculate trapping effort by station and occasion
@@ -234,44 +196,32 @@ spatialDetectionHistory <- function(recordTableIndividual,
 
   ###################
   # remove records that fall into buffer period or were taken after maxNumberDays
-  arg.list0 <- list(subset_species2 = subset_species, stationCol2 = stationCol, date_ranges2 = date_ranges, occasionStartTime2 = occasionStartTime, timeZone2 = timeZone)
-
-  if(!hasArg(maxNumberDays) & !hasArg(buffer))  subset_species <- do.call(cleanSubsetSpecies, arg.list0)
-  if(hasArg(maxNumberDays)  & !hasArg(buffer))  subset_species <- do.call(cleanSubsetSpecies, c(arg.list0, maxNumberDays = maxNumberDays))
-  if(!hasArg(maxNumberDays) & hasArg(buffer))   subset_species <- do.call(cleanSubsetSpecies, c(arg.list0, buffer =  buffer))
-  if(hasArg(maxNumberDays)  & hasArg(buffer))   subset_species <- do.call(cleanSubsetSpecies, c(arg.list0, maxNumberDays = maxNumberDays, buffer = buffer))
-
-  rm(arg.list0)
 
+  subset_species <- cleanSubsetSpecies(subset_species2 = subset_species, stationCol2 = stationCol, date_ranges2 = date_ranges)
 
   ############
   #  define the 1st day of the effective survey period.
 
-  if(day1 == "survey")  {    # same starting day for all stations
-    time2 = rep(as.POSIXct(paste(as.character(min(date_ranges$cam.min)), "00:00:00"), tz = timeZone), times = nrow(subset_species))
-  } else {
-    if(day1 == "station") {  # individual starting days for all stations
-      time2 = as.POSIXct(paste(as.character(date_ranges$cam.min[match(subset_species[,stationCol], rownames(date_ranges))]), "00:00:00"), tz = timeZone)
-    } else {
-      if(class(as.Date(day1)) == "Date") {   #  some specific date
-        time2 = rep(as.POSIXct( paste(as.Date(day1), "00:00:00"), tz = timeZone), times = nrow(subset_species))
-      }
-    }
-  }
+if(day1 %in% c("survey")){
+  time2 <- date_ranges$start_first_occasion_survey[match(subset_species[,stationCol], rownames(date_ranges))]
+} else {
+  time2 <- date_ranges$start_first_occasion[match(subset_species[,stationCol], rownames(date_ranges))]
+}
 
 
+#  time2 <- date_ranges$start_first_occasion[match(subset_species[,stationCol], rownames(date_ranges))]
+
   # calculate time difference between records and first day of detection history (the occasion each record belongs into)
   occasionCol <- "Occasion"
 
-  subset_species[,occasionCol] <- as.numeric(ceiling((difftime(time1 = subset_species[,recordDateTimeCol],
+  subset_species[,occasionCol] <- as.numeric(ceiling((difftime(time1  = subset_species$DateTime2,
                                                           time2 =  time2,
                                                           units = "secs",
-                                                          tz    = timeZone)
-                                                 - occasionStartTime * 3600) / (occasionLength * 86400)))
+                                                          tz = timeZone)
+                                                / (occasionLength * 86400))))
 
 
-
-  if(max(subset_species[,occasionCol]) > ncol(effort)) {stop("encountered a bug. Sorry. Please report it.")}
+  if(max(subset_species[,occasionCol]) > ncol(effort)) {stop("encountered a bug. I'm Sorry. Please report it.")}
 
 
   # column names for effort matrix
@@ -299,7 +249,7 @@ spatialDetectionHistory <- function(recordTableIndividual,
 # add required session column
   sdh1 <- data.frame(Session = 1, sdh0)   
 
-# if requested, make remove duplicate records (makes capthist binary. otherwise it returns counts)
+# if requested,  remove duplicate records (makes capthist binary. otherwise it returns counts)
   if(output == "binary"){
     sdh2 <- unique(sdh1)                  # remove duplicate rows
   } else {
